package main

import (
	p "code.ivysaur.me/plexus"
)

const (
	RULE_COMMENT_LINE = iota
	RULE_QUOTED_LITERAL
	RULE_OR
	RULE_IDENTIFIER
	RULE_DECLARATION

	RULE_SINGLE_ARG_ELEMENT_BASIC
	RULE_SINGLE_ARG_ELEMENT_REPEATABLE
	RULE_SINGLE_ARG_ELEMENT

	RULE_ARGLIST          // 8
	RULE_ARGLIST_OPTIONAL // 9
	RULE_ARGLIST_NESTED   // 10

	RULE_EXPRESSION // 11

	RULE_DOCUMENT_ENTRY
	RULE_DOCUMENT
)

func ParseBNF(input []byte) (*p.MatchNode, error) {

	pa := p.NewParseAttempt()
	pa.Syntax = p.SyntaxTree{
		RULE_COMMENT_LINE:   []p.MatchNode{p.Regexp("#[^\n]*\n+")},
		RULE_QUOTED_LITERAL: []p.MatchNode{p.Regexp("'[^']*'")},
		RULE_OR:             []p.MatchNode{p.Literal("|")},
		RULE_IDENTIFIER:     []p.MatchNode{p.Regexp("[a-zA-Z_]+"), p.Ref(p.RT_RULE_NONE_OF_THE_REMAINING), p.Literal(":")},
		RULE_DECLARATION:    []p.MatchNode{p.Regexp("[a-zA-Z_]+"), p.Literal(":")},

		RULE_SINGLE_ARG_ELEMENT_BASIC:      []p.MatchNode{p.Ref(p.RT_RULE_ONE_OF_THE_REMAINING), p.Ref(RULE_QUOTED_LITERAL), p.Ref(RULE_IDENTIFIER), p.Ref(RULE_ARGLIST_OPTIONAL), p.Ref(RULE_ARGLIST_NESTED)},
		RULE_SINGLE_ARG_ELEMENT_REPEATABLE: []p.MatchNode{p.Ref(RULE_SINGLE_ARG_ELEMENT_BASIC), p.Ref(p.RT_RULE_OPTIONAL_REMAINING), p.Regexp("[+*]")},
		RULE_SINGLE_ARG_ELEMENT:            []p.MatchNode{p.Ref(p.RT_RULE_ONE_OF_THE_REMAINING), p.Ref(RULE_OR), p.Ref(RULE_SINGLE_ARG_ELEMENT_REPEATABLE)},

		RULE_ARGLIST:          []p.MatchNode{p.Ref(p.RT_RULE_REPEAT_NEXT_ONEPLUS), p.Ref(RULE_SINGLE_ARG_ELEMENT)},
		RULE_ARGLIST_OPTIONAL: []p.MatchNode{p.Literal("["), p.Ref(RULE_ARGLIST), p.Literal("]")},
		RULE_ARGLIST_NESTED:   []p.MatchNode{p.Literal("("), p.Ref(RULE_ARGLIST), p.Literal(")")},

		RULE_EXPRESSION: []p.MatchNode{p.Ref(RULE_DECLARATION), p.Ref(RULE_ARGLIST)},

		RULE_DOCUMENT_ENTRY: []p.MatchNode{p.Ref(p.RT_RULE_ONE_OF_THE_REMAINING), p.Ref(RULE_COMMENT_LINE), p.Ref(RULE_EXPRESSION)},
		RULE_DOCUMENT:       []p.MatchNode{ /*p.SkipWhitespace(" \t\r\n"),*/ p.Ref(p.RT_RULE_REPEAT_NEXT_ONEPLUS), p.Ref(RULE_DOCUMENT_ENTRY)},
	}

	mn, err := pa.Parse(RULE_DOCUMENT, input)
	if err != nil {
		return nil, err
	}

	return mn, nil
}
