package plexus

import (
	"testing"
)

const (
	RULE_PLUS           = iota // 0
	RULE_MINUS                 // 1
	RULE_NUMBER                // 2
	RULE_INFIX_OPERATOR        // 3
	RULE_EXPRESSION            // 4
)

func TestParse(t *testing.T) {

	pa := NewParseAttempt()
	pa.Syntax = SyntaxTree{
		RULE_PLUS:           []MatchNode{Literal("+")},
		RULE_MINUS:          []MatchNode{Literal("-")},
		RULE_NUMBER:         []MatchNode{Regexp("[0-9]+")},
		RULE_INFIX_OPERATOR: []MatchNode{Ref(RT_RULE_ONE_OF_THE_REMAINING), Ref(RULE_PLUS), Ref(RULE_MINUS)},
		RULE_EXPRESSION:     []MatchNode{Ref(RULE_NUMBER), Ref(RULE_INFIX_OPERATOR), Ref(RULE_NUMBER)},
	}

	mn, err := pa.Parse(RULE_EXPRESSION, []byte(" 3 + 4 "))

	if err != nil {
		t.Log(err.Error())
		t.Fail()
	}

	expected := MatchNode{
		Type: RULE_EXPRESSION,
		Children: []MatchNode{
			MatchNode{
				Type:     RULE_NUMBER,
				Children: []MatchNode{Literal("3")},
			},
			MatchNode{
				Type: RULE_INFIX_OPERATOR,
				Children: []MatchNode{
					MatchNode{
						Type:     RULE_PLUS,
						Children: []MatchNode{Literal("+")},
					},
				},
			},
			MatchNode{
				Type:     RULE_NUMBER,
				Children: []MatchNode{Literal("4")},
			},
		},
	}

	if !mn.Equals(&expected) {
		t.Log("Unexpected parse result")
		t.Fail()
	}
}
